<?php

namespace TYPO3\Election\Controller;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
use TYPO3\Election\Domain\Model\Dto\ElectionVoting;
use TYPO3\Election\Domain\Model\ElectionInvitation;
use TYPO3\Election\Domain\Model\ElectionVote;
use TYPO3\Election\Domain\Model\Election;

/**
 * Class FeElectionController
 */
class FeElectionController extends ActionController
{
    const CONTROLLER_NAME = 'FeElection';
    const ACTION_VOTE = 'vote';
    const ACTION_SHOW = 'show';
    const ACTION_SUBMIT_VOTE = 'submitVote';

    /**
     * @var \TYPO3\CMS\Extbase\Security\Cryptography\HashService
     * @inject
     */
    protected $hashService = null;

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectionVoteRepository
     * @inject
     */
    protected $electionVoteRepository = null;

    /**
     * @var \TYPO3\Election\Domain\Repository\ElectionInvitationRepository
     * @inject
     */
    protected $electionInvitationRepository = null;

    /**
     * @throws \TYPO3\CMS\Extbase\Mvc\Exception\InvalidArgumentNameException
     * @throws \TYPO3\CMS\Extbase\Mvc\Exception\NoSuchArgumentException
     */
    public function initializeVoteAction()
    {
        if ($this->request->hasArgument('electionInvitation')) {
            $electionInvitation = $this->request->getArgument('electionInvitation');
            if ($electionInvitation === '-1') {
                $this->response->appendContent('TEST SUCCESSFUL');
                $this->request->setArgument('electionInvitation', null);
                $this->actionMethodName = 'voidAction';
            }
        }
    }

    /**
     * @param ElectionInvitation $electionInvitation
     * @param string $hmac
     * @return string
     */
    public function voteAction(ElectionInvitation $electionInvitation = null, $hmac = '')
    {
        if (null !== $electionInvitation && '' !== $hmac) {
            $saltedEmail = $electionInvitation->getSecret() . $electionInvitation->getElector()->getEmail();
            if ($this->hashService->validateHmac($saltedEmail, $hmac)) {
                if ($electionInvitation->isVoted()) {
                    $this->addFlashMessage(
                        LocalizationUtility::translate('controller.fe.election.vote.already_voted', 'election'),
                        LocalizationUtility::translate('controller.fe.election.vote.request_failed', 'election'),
                        AbstractMessage::ERROR
                    );
                    $this->redirect(FeDashboardController::ACTION_INDEX, FeDashboardController::CONTROLLER_NAME);
                } elseif ($electionInvitation->getElectionEndDateAsTimestamp() < time()) {
                    $this->addFlashMessage(
                        LocalizationUtility::translate('controller.fe.election.vote.election_finished', 'election'),
                        LocalizationUtility::translate('controller.fe.election.vote.request_failed', 'election'),
                        AbstractMessage::ERROR
                    );
                    $this->redirect(FeDashboardController::ACTION_INDEX, FeDashboardController::CONTROLLER_NAME);
                } else {
                    $this->view->assign('electionInvitation', $electionInvitation);
                    $this->view->assign('electionVoting', new ElectionVoting());
                }
            } else {
                $this->addFlashMessage(
                    LocalizationUtility::translate('controller.fe.election.vote.hmac_invalid', 'election'),
                    LocalizationUtility::translate('controller.fe.election.vote.request_failed', 'election'),
                    AbstractMessage::ERROR
                );
                $this->redirect(FeDashboardController::ACTION_INDEX, FeDashboardController::CONTROLLER_NAME);
            }
        } else {
            $this->addFlashMessage(
                LocalizationUtility::translate('controller.fe.election.vote.no_election_or_hmac', 'election'),
                LocalizationUtility::translate('controller.fe.election.vote.request_failed', 'election'),
                AbstractMessage::ERROR
            );
            $this->redirect(FeDashboardController::ACTION_INDEX, FeDashboardController::CONTROLLER_NAME);
        }
    }

    /**
     * @throws \TYPO3\CMS\Extbase\Mvc\Exception\InvalidArgumentNameException
     * @throws \TYPO3\CMS\Extbase\Mvc\Exception\NoSuchArgumentException
     */
    public function initializeSubmitVoteAction()
    {
        if ($this->request->hasArgument('electionVoting')) {
            $electionVoting = $this->request->getArgument('electionVoting');
            if (isset($electionVoting['nominees']) && is_array($electionVoting['nominees'])) {
                $electionVoting['nominees'] = array_filter($electionVoting['nominees']);
                $this->request->setArgument('electionVoting', $electionVoting);
            }
        }
    }

    /**
     * @param ElectionVoting $electionVoting
     * @validate $electionVoting \TYPO3\Election\Domain\Validator\ElectionVotingValidator
     */
    public function submitVoteAction(ElectionVoting $electionVoting)
    {
        $electionInvitation = $electionVoting->getElectionInvitation();
        if ($electionInvitation->isVoted()) {
            $this->addFlashMessage(
                LocalizationUtility::translate('controller.fe.election.vote.already_voted', 'election'),
                LocalizationUtility::translate('controller.fe.election.vote.request_failed', 'election'),
                AbstractMessage::ERROR
            );
            $this->redirect(FeDashboardController::ACTION_INDEX, FeDashboardController::CONTROLLER_NAME);
        }
        $electionInvitation->setVoted(true);
        $this->electionInvitationRepository->update($electionInvitation);

        foreach ($electionVoting->getNominees() as $electedNominee) {
            $electionVote = $this->objectManager->get(ElectionVote::class);
            $electionVote->setElection($electionInvitation->getElectionCircular()->getElection());
            $electionVote->setElector($electionInvitation->getElector());
            $electionVote->setNominee($electedNominee);
            $this->electionVoteRepository->add($electionVote);
        }

        $this->addFlashMessage(
            LocalizationUtility::translate('controller.fe.election.vote.thank_you', 'election'),
            LocalizationUtility::translate('controller.fe.election.vote.success', 'election'),
            AbstractMessage::OK
        );
        $this->redirect(FeDashboardController::ACTION_INDEX, FeDashboardController::CONTROLLER_NAME);
    }

    /**
     * @param Election $election
     */
    public function showAction(Election $election)
    {
        $this->view->assign('election', $election);
        $this->view->assign('results', $election->getElectedNomineesAndVotes());
    }


    /**
     * @return string
     */
    public static function getActionsForPluginConfiguration()
    {
        return implode(',', [self::ACTION_VOTE, self::ACTION_SUBMIT_VOTE, self::ACTION_SHOW]);

    }

    /**
     *
     */
    public function voidAction()
    {
        return '';
    }

    /**
     * A template method for displaying custom error flash messages, or to
     * display no flash message at all on errors. Override this to customize
     * the flash message in your action controller.
     *
     * @return string|boolean The flash message or FALSE if no flash message should be set
     * @api
     */
    protected function getErrorFlashMessage()
    {

        return LocalizationUtility::translate(get_class($this) . '->' . $this->actionMethodName . '()', 'election');
    }
}
